import json

import mock

from Tests.test_api.base import RestAPITestBase


class TestSettingsView(RestAPITestBase):

    def test_get_mute(self):
        """
        Test for api get mute.
        """
        url = self.get_server_url() + "/settings/mute"
        headers = {"Content-Type": "application/json"}

        self.settings.options.mute = False
        result = self.client.get(url, headers=headers)

        expected_content = {
            "mute": False
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_mute(self):
        """
        Test for api set mute.
        """
        url = self.get_server_url() + "/settings/mute"
        data = {"mute": "True"}
        headers = {"Content-Type": "application/json"}

        self.settings.options.mute = False
        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "mute": True
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_get_deaf(self):
        """
        Test for api get deaf.
        """
        url = self.get_server_url() + "/settings/deaf"
        headers = {"Content-Type": "application/json"}

        self.settings.options.deaf = False
        result = self.client.get(url, headers=headers)

        expected_content = {
            "deaf": False
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_deaf(self):
        """
        Test for api set deaf.
        """
        url = self.get_server_url() + "/settings/deaf"
        data = {"deaf": "True"}
        headers = {"Content-Type": "application/json"}

        self.settings.options.deaf = False
        with mock.patch("kalliope.core.SignalLauncher.SignalLauncher.get_order_instance") as mock_order:
            result = self.client.post(url,
                                      headers=headers,
                                      data=json.dumps(data))

            expected_content = {
                "deaf": True
            }

            mock_order.assert_called_once()
            self.assertEqual(json.dumps(expected_content, sort_keys=True),
                             json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
            self.assertEqual(result.status_code, 200)

    def test_get_recognizer_multiplier(self):
        """
        Test for api get recognizer_multiplier.
        """
        url = self.get_server_url() + "/settings/recognizer_multiplier"
        headers = {"Content-Type": "application/json"}
        self.settings.options.recognizer_multiplier = 3000
        result = self.client.get(url, headers=headers)

        expected_content = {
            "recognizer_multiplier": 3000
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_recognizer_multiplier(self):
        """
        Test for api set recognizer_multiplier.
        """
        url = self.get_server_url() + "/settings/recognizer_multiplier"
        data = {"recognizer_multiplier": "6000"}
        headers = {"Content-Type": "application/json"}

        self.settings.recognizer_multiplier = 4000
        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "recognizer_multiplier": "6000"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_get_recognizer_energy_ratio(self):
        """
        Test for api get recognizer_energy_ratio second.
        """
        url = self.get_server_url() + "/settings/recognizer_energy_ratio"
        headers = {"Content-Type": "application/json"}
        self.settings.options.recognizer_energy_ratio = 30
        result = self.client.get(url, headers=headers)

        expected_content = {
            "recognizer_energy_ratio": 30
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_recognizer_energy_ratio(self):
        """
        Test for api set recognizer_energy_ratio.
        """
        url = self.get_server_url() + "/settings/recognizer_energy_ratio"
        data = {"recognizer_energy_ratio": "60"}
        headers = {"Content-Type": "application/json"}

        self.settings.recognizer_energy_ratio = 40
        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "recognizer_energy_ratio": "60"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_get_recognizer_recording_timeout(self):
        """
        Test for api get recognizer_recording_timeout.
        """
        url = self.get_server_url() + "/settings/recognizer_recording_timeout"
        headers = {"Content-Type": "application/json"}
        self.settings.options.recognizer_recording_timeout = 3000
        result = self.client.get(url, headers=headers)

        expected_content = {
            "recognizer_recording_timeout": 3000
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_recognizer_recording_timeout(self):
        """
        Test for api set recognizer_recording_timeout.
        """
        url = self.get_server_url() + "/settings/recognizer_recording_timeout"
        data = {"recognizer_recording_timeout": "6000"}
        headers = {"Content-Type": "application/json"}

        self.settings.recognizer_recording_timeout = 4000
        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "recognizer_recording_timeout": "6000"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_get_recognizer_recording_timeout_with_silence(self):
        """
        Test for api get recognizer_recording_timeout_with_silence second.
        """
        url = self.get_server_url() + "/settings/recognizer_recording_timeout_with_silence"
        headers = {"Content-Type": "application/json"}
        self.settings.options.recognizer_recording_timeout_with_silence = 30
        result = self.client.get(url, headers=headers)

        expected_content = {
            "recognizer_recording_timeout_with_silence": 30
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_recognizer_recording_timeout_with_silence(self):
        """
        Test for api set recognizer_recording_timeout_with_silence.
        """
        url = self.get_server_url() + "/settings/recognizer_recording_timeout_with_silence"
        data = {"recognizer_recording_timeout_with_silence": "60"}
        headers = {"Content-Type": "application/json"}

        self.settings.recognizer_recording_timeout_with_silence = 40
        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "recognizer_recording_timeout_with_silence": "60"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)
        
    def test_get_default_tts(self):
        """
        Test for api get default tts name.
        """
        url = self.get_server_url() + "/settings/default_tts"
        headers = {"Content-Type": "application/json"}
        self.settings.default_tts_name = "test"
        result = self.client.get(url, headers=headers)

        expected_content = {
            "default_tts": "test"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_default_tts(self):
        """
        Test for api set default_tts.
        """
        url = self.get_server_url() + "/settings/default_tts"
        data = {"default_tts": "pico2wave"}
        headers = {"Content-Type": "application/json"}

        self.settings.default_tts_name = "test"
        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "default_tts": "pico2wave"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_get_default_stt(self):
        """
        Test for api get default stt name.
        """
        url = self.get_server_url() + "/settings/default_stt"
        headers = {"Content-Type": "application/json"}
        self.settings.default_stt_name = "test"
        result = self.client.get(url, headers=headers)

        expected_content = {
            "default_stt": "test"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_default_stt(self):
        """
        Test for api set default_stt.
        """
        url = self.get_server_url() + "/settings/default_stt"
        data = {"default_stt": "google"}
        headers = {"Content-Type": "application/json"}

        self.settings.default_stt_name = "test"
        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "default_stt": "google"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_get_default_player(self):
        """
        Test for api get default player.
        """
        url = self.get_server_url() + "/settings/default_player"
        headers = {"Content-Type": "application/json"}
        self.settings.default_player_name = "test"
        result = self.client.get(url, headers=headers)

        expected_content = {
            "default_player": "test"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_default_player(self):
        """
        Test for api set default_player.
        """
        url = self.get_server_url() + "/settings/default_player"
        data = {"default_player": "mplayer"}
        headers = {"Content-Type": "application/json"}

        self.settings.default_player_name = "test"
        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "default_player": "mplayer"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_get_default_trigger(self):
        """
        Test for api get default trigger.
        """
        url = self.get_server_url() + "/settings/default_trigger"
        headers = {"Content-Type": "application/json"}
        self.settings.default_trigger_name = "test"
        result = self.client.get(url, headers=headers)

        expected_content = {
            "default_trigger": "test"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_default_trigger(self):
        """
        Test for api set default_trigger.
        """
        url = self.get_server_url() + "/settings/default_trigger"
        data = {"default_trigger": "snowboy"}
        headers = {"Content-Type": "application/json"}

        self.settings.default_trigger_name = "test"
        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "default_trigger": "snowboy"
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_get_hooks(self):
        """
        Test for api get hooks.
        """
        url = self.get_server_url() + "/settings/hooks"
        headers = {"Content-Type": "application/json"}
        self.settings.hooks = {
            "on_start": "on-start-synapse",
            "on_waiting_for_trigger": {},
            "on_triggered": "on-triggered-synapse",
            "on_start_listening": {},
            "on_stop_listening": {},
            "on_order_found": {},
            "on_order_not_found": "order-not-found-synapse",
            "on_processed_synapses": {},
            "on_deaf": {},
            "on_undeaf": {},
            "on_start_speaking": {},
            "on_stop_speaking": {},
            "on_stt_error": {}
        }
        result = self.client.get(url, headers=headers)

        expected_content = {
            "hooks": {
                "on_start": "on-start-synapse",
                "on_waiting_for_trigger": {},
                "on_triggered": "on-triggered-synapse",
                "on_start_listening": {},
                "on_stop_listening": {},
                "on_order_found": {},
                "on_order_not_found": "order-not-found-synapse",
                "on_processed_synapses": {},
                "on_deaf": {},
                "on_undeaf": {},
                "on_start_speaking": {},
                "on_stop_speaking": {},
                "on_stt_error": {}
            }
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_hooks(self):
        """
        Test for api set hooks.
        """
        url = self.get_server_url() + "/settings/hooks"
        data = {"on_waiting_for_trigger": "synapse"}
        headers = {"Content-Type": "application/json"}

        self.settings.hooks = {
            "on_start": "on-start-synapse",
            "on_waiting_for_trigger": {},
            "on_triggered": "on-triggered-synapse",
            "on_start_listening": {},
            "on_stop_listening": {},
            "on_order_found": {},
            "on_order_not_found": "order-not-found-synapse",
            "on_processed_synapses": {},
            "on_deaf": {},
            "on_undeaf": {},
            "on_start_speaking": {},
            "on_stop_speaking": {},
            "on_stt_error": {}
        }

        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "hooks": {
                "on_start": "on-start-synapse",
                "on_waiting_for_trigger": "synapse",
                "on_triggered": "on-triggered-synapse",
                "on_start_listening": {},
                "on_stop_listening": {},
                "on_order_found": {},
                "on_order_not_found": "order-not-found-synapse",
                "on_processed_synapses": {},
                "on_deaf": {},
                "on_undeaf": {},
                "on_start_speaking": {},
                "on_stop_speaking": {},
                "on_stt_error": {}
            }
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_get_variables(self):
        """
        Test for api get variables.
        """
        url = self.get_server_url() + "/settings/variables"
        headers = {"Content-Type": "application/json"}
        self.settings.variables = {
            "test": "tust",
            "tost": "tist"
        }
        result = self.client.get(url, headers=headers)

        expected_content = {
            "variables": {
                "test": "tust",
                "tost": "tist"
            }
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)

    def test_set_variables(self):
        """
        Test for api set variables.
        """
        url = self.get_server_url() + "/settings/variables"
        data = {"toto": "titi"}
        headers = {"Content-Type": "application/json"}

        self.settings.variables = {
            "tt": "aa",
            "uu": "ii"
        }

        result = self.client.post(url,
                                  headers=headers,
                                  data=json.dumps(data))

        expected_content = {
            "variables": {
                "tt": "aa",
                "uu": "ii",
                "toto": "titi"
            }
        }

        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(result.get_data().decode('utf-8')), sort_keys=True))
        self.assertEqual(result.status_code, 200)



# coding: utf8
import os
import unittest

from Tests.utils.utils import get_test_path
from kalliope.core.Models import Singleton, Signal

from kalliope.core.ConfigurationManager import BrainLoader
from kalliope.core.Models import Neuron
from kalliope.core.Models import Synapse
from kalliope.core.Models.Brain import Brain
from kalliope.core.Models.settings.Settings import Settings


class TestBrainLoader(unittest.TestCase):

    def setUp(self):
        # be sure the brain haven't been instantiated before
        Singleton._instances = dict()
        self.brain_to_test = get_test_path("brains/brain_test.yml")

        self.expected_result = [
            {'signals': [{'order': 'test_order'}],
             'neurons': [{'say': {'message': ['test message']}}],
             'name': 'test'},
            {'signals': [{'order': 'test_order_2'}],
             'neurons': [{'say': {'message': ['test message']}}],
             'name': 'test2'},
            {'signals': [{'order': 'order_for_int'}],
             'neurons': [{'sleep': {'seconds': 60}}],
             'name': 'testint'},
            {'includes': ['included_brain_test.yml']},
            {'signals': [{'order': 'test_order_3'}],
             'neurons': [{'say': {'message': ['test message']}}],
             'name': 'test3'}
        ]

    def tearDown(self):
        Singleton._instances = dict()

    def test_get_yaml_config(self):
        """
        Test we can get a yaml config from the path
        """
        brain_loader = BrainLoader(file_path=self.brain_to_test)
        self.assertEqual(brain_loader.yaml_config, self.expected_result)

    def test_load_brain(self):
        """
        Test the class return a valid brain object
        """

        neuron = Neuron(name='say', parameters={'message': ['test message']})
        neuron2 = Neuron(name='sleep', parameters={'seconds': 60})

        signal1 = Signal(name="order", parameters="test_order")
        signal2 = Signal(name="order", parameters="test_order_2")
        signal3 = Signal(name="order", parameters="test_order_3")
        signal4 = Signal(name="order", parameters="order_for_int")

        synapse1 = Synapse(name="test", neurons=[neuron], signals=[signal1])
        synapse2 = Synapse(name="test2", neurons=[neuron], signals=[signal2])
        synapse3 = Synapse(name="test3", neurons=[neuron], signals=[signal3])
        synapse4 = Synapse(name="testint", neurons=[neuron2], signals=[signal4])
        synapses = [synapse1, synapse2, synapse4, synapse3]

        brain = Brain()
        brain.synapses = synapses
        brain.brain_file = self.brain_to_test
        brain.brain_yaml = self.expected_result

        brain_loader = BrainLoader(file_path=self.brain_to_test)
        self.assertEqual(brain, brain_loader.brain)

    def test_get_neurons(self):
        """
        Test to get neurons from the brainLoader
        scenarii:
            - 1/ get a simple neuron from the brainloader
            - 2/ get a neuron with brackets
            - 3/ get a neuron with int as parameters
        """
        # 1/ get a simple neuron from the brainloader
        st = Settings()
        neuron_list = [{'say': {'message': ['test message']}}]

        neuron = Neuron(name='say', parameters={'message': ['test message']})

        bl = BrainLoader(file_path=self.brain_to_test)
        neurons_from_brain_loader = bl.get_neurons(neuron_list,
                                                   settings=st)

        self.assertEqual([neuron], neurons_from_brain_loader)

        # 2/ get a neuron with global variables as parameters
        neuron_list = [{'say': {'message': ['bonjour {{name}}']}}]

        st = Settings()
        bl = BrainLoader(file_path=self.brain_to_test)
        neurons_from_brain_loader = bl.get_neurons(neuron_list,
                                                   settings=st)

        neuron = Neuron(name='say', parameters={'message': ['bonjour {{name}}']})

        self.assertEqual([neuron], neurons_from_brain_loader)

        # 3/ get a neuron with int as parameters
        st = Settings()
        neuron_list = [{'sleep': {'seconds': 60}}]

        neuron = Neuron(name='sleep', parameters={'seconds': 60})

        bl = BrainLoader(file_path=self.brain_to_test)
        neurons_from_brain_loader = bl.get_neurons(neuron_list,
                                                   settings=st)

        self.assertEqual([neuron], neurons_from_brain_loader)

    def test_get_signals(self):
        signals = [{'order': 'test_order'}]

        signal = Signal(name="order", parameters="test_order")

        bl = BrainLoader(file_path=self.brain_to_test)
        signals_from_brain_loader = bl.get_signals(signals)

        self.assertEqual([signal], signals_from_brain_loader)

    def test_singleton(self):
        bl1 = BrainLoader(file_path=self.brain_to_test)
        bl2 = BrainLoader(file_path=self.brain_to_test)

        self.assertTrue(bl1.brain is bl2.brain)


if __name__ == '__main__':
    unittest.main()



import os

from flask import Flask
from flask_testing import LiveServerTestCase

from kalliope.core import LifoManager
from kalliope.core.ConfigurationManager import BrainLoader
from kalliope.core.ConfigurationManager import SettingLoader
from kalliope.core.Models import Singleton
from kalliope.core.RestAPI.FlaskAPI import FlaskAPI


class RestAPITestBase(LiveServerTestCase):

    def tearDown(self):
        Singleton._instances = {}
        # clean the lifo
        LifoManager.clean_saved_lifo()

    def create_app(self):
        """
        executed once at the beginning of the test
        """
        # be sure that the singleton haven't been loaded before
        Singleton._instances = {}
        current_path = os.getcwd()
        if "/Tests" in current_path:
            full_path_brain_to_test = current_path + os.sep + "brains/brain_test_api.yml"
            self.audio_file = "files/bonjour.wav"
        else:
            full_path_brain_to_test = current_path + os.sep + "Tests/brains/brain_test_api.yml"
            self.audio_file = "Tests/files/bonjour.wav"

        # rest api config
        self.sl = SettingLoader()
        self.settings = self.sl.settings
        self.settings.rest_api.password_protected = False
        self.settings.active = True
        self.settings.port = 5000
        self.settings.allowed_cors_origin = "*"
        self.settings.default_synapse = None
        self.settings.hooks["on_order_not_found"] = "order-not-found-synapse"

        # prepare a test brain
        brain_to_test = full_path_brain_to_test
        brain_loader = BrainLoader(file_path=brain_to_test)
        brain = brain_loader.brain

        self.app = Flask(__name__)
        self.app.config['TESTING'] = True
        self.flask_api = FlaskAPI(self.app, port=5000, brain=brain)
        self.client = self.app.test_client()
        return self.flask_api.app



import glob
import inspect
import logging
import os

from flask import jsonify, Blueprint, request
from kalliope.core.ResourcesManager import ResourcesManager, ResourcesManagerException
from kalliope.core.Utils import Utils

logging.basicConfig()
logger = logging.getLogger("kalliope")

LIST_EXCLUDED_DIR_NAME = ["__pycache__"]
KALLIOPE_WEBSITE_NEURON_URL = "https://raw.githubusercontent.com/kalliope-project/kalliope-project.github.io/" \
                              "sources/_data/community_neurons.yml"


class NeuronsView(Blueprint):
    def __init__(self, name, import_name, app=None, brain=None, settings=None):
        self.brain = brain
        self.settings = settings
        self.app = app
        super(NeuronsView, self).__init__(name, import_name)

        # routes
        self.add_url_rule('/neurons', view_func=self.get_neurons, methods=['GET'])
        self.add_url_rule('/neurons/install', view_func=self.install_resource_by_git_url, methods=['POST'])

    def get_neurons(self):
        """
        list all installed neuron

        curl -i --user admin:secret -X GET http://127.0.0.1:5000/neurons
        :return:
        """
        data = {
            "core": self._get_list_core_neuron(),
            "community": self._get_list_installed_community_neuron()
        }
        logger.debug("[FlaskAPI] get_neurons: all")
        data = jsonify(data)
        return data, 200

    @staticmethod
    def install_resource_by_git_url():
        """
        Install a new resource from the given git URL. Call the resource manager

        curl -i -H "Content-Type: application/json" \
        --user admin:secret \
        -X POST \
        -d '
        {
            "git_url": "https://github.com/kalliope-project/kalliope_neuron_wikipedia.git",
            "sudo_password": "azerty"  # TODO add kalliope to sudoer file and remove this
        }
        ' \
        http://127.0.0.1:5000/neurons/install
        """
        if not request.get_json() or 'git_url' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'git_url' not set"
            }
            return jsonify(error=data), 400

        parameters = {
            "git_url": request.get_json()["git_url"],
            "sudo_password": request.get_json()["sudo_password"]
        }
        res_manager = ResourcesManager(**parameters)
        try:
            dna = res_manager.install()
        except ResourcesManagerException as e:
            data = {
                "error": "%s" % e
            }
            return jsonify(data), 400

        if dna is not None:
            return jsonify(dna.serialize()), 200
        else:
            data = {
                "Error": "Error during resource installation"
            }
            return jsonify(error=data), 400

    def _get_list_core_neuron(self):
        current_path = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
        kalliope_core_neuron_folder = Utils.get_current_file_parent_path(Utils.get_current_file_parent_parent_path(current_path)) + os.sep + "neurons/*"
        return self._get_neuron_name_list_from_path(kalliope_core_neuron_folder)

    def _get_list_installed_community_neuron(self):
        if self.settings.resources.neuron_folder is not None:
            return self._get_neuron_name_list_from_path(self.settings.resources.neuron_folder + os.sep + "/*")

        return list()

    @staticmethod
    def _get_neuron_name_list_from_path(neuron_path_list):
        """
        From a given path, return all folder name as list.
        E.g: /home/pi/kalliope/neurons
        Will return [wikipedia_searcher, gmail]
        :param neuron_path_list: path to the community neurons folder
        :return:
        """

        glob_neuron_path_list = glob.glob(neuron_path_list)

        neuron_name_list = list()
        for neuron_path in glob_neuron_path_list:
            if os.path.isdir(neuron_path):
                neuron_name = os.path.basename(os.path.normpath(neuron_path))
                if neuron_name not in LIST_EXCLUDED_DIR_NAME:
                    neuron_name_list.append(neuron_name)
        return neuron_name_list



import os
import unittest
from unittest import mock


def get_test_path(file_path):
    """
    return the path of a file with "Tests" depending of the current location of the execution
    :return: string path
    """
    current_path = os.getcwd()
    if "/Tests" in current_path:
        return current_path + os.sep + file_path
    else:
        return current_path + os.sep + "Tests" + os.sep + file_path


class TestTestUtils(unittest.TestCase):

    def test_get_test_path(self):
        # Tests is in path
        with mock.patch('Tests.utils.utils.os.getcwd', return_value='/home/user/Documents/kalliope/Tests'):
            expected = "/home/user/Documents/kalliope/Tests/file"
            self.assertEqual(expected, get_test_path("file"))

        # Tests not in path
        with mock.patch('Tests.utils.utils.os.getcwd', return_value='/home/user/Documents/kalliope'):
            expected = "/home/user/Documents/kalliope/Tests/file"
            self.assertEqual(expected, get_test_path("file"))



import logging
from functools import wraps
from flask import request, Response

from kalliope import Utils
from kalliope.core.ConfigurationManager import SettingLoader

logging.basicConfig()
logger = logging.getLogger("kalliope")


def check_auth(username, password):
    """This function is called to check if a username /
    password combination is valid.
    """
    sl = SettingLoader()
    settings = sl.settings
    return username == settings.rest_api.login and password == settings.rest_api.password


def authenticate():
    """Sends a 401 response that enables basic auth"""
    return Response(
        'Could not verify your access level for that URL.\n'
        'You have to login with proper credentials', 401,
        {'WWW-Authenticate': 'Basic realm="Login Required"'})


def requires_auth(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        sl = SettingLoader()
        settings = sl.settings
        if settings.rest_api.password_protected:
            auth = request.authorization
            if not auth or not check_auth(auth.username, auth.password):
                return authenticate()
        return f(*args, **kwargs)
    return decorated


def get_parameters_from_request(http_request):
    """
    Get "parameters" object from the http_request
    :param http_request: flask http request
    :return: dict of parameters from the received json
    """
    parameters = None
    try:
        # Silent True in case no parameters it does not raise an error
        received_json = http_request.get_json(silent=True, force=True)
        if 'parameters' in received_json:
            parameters = received_json['parameters']
    except TypeError:
        logger.debug("[FlaskAPI] no parameters received in http request")
        pass
    return parameters


def get_value_flag_from_request(http_request, flag_to_find, is_boolean=False):
    """
    Get the value flag from the request if exist, None otherwise !
    :param http_request:
    :param flag_to_find: json flag to find in the http_request
    :param is_boolean: True if the expected value is a boolean, False Otherwise.
    :return: the Value of the flag that has been found in the request
    """
    flag_value = None
    try:
        received_json = http_request.get_json(force=True, silent=True, cache=True)
        if flag_to_find in received_json:
            flag_value = received_json[flag_to_find]
            if is_boolean:
                flag_value = Utils.str_to_bool(flag_value)
    except TypeError:
        # no json received
        pass
    return flag_value



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re

import pip
from setuptools import setup, find_packages
from codecs import open
from os import path

basedir = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(basedir, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


# locate our version number
def read_version_py(file_name):
    try:
        version_string_line = open(file_name, "rt").read()
    except EnvironmentError:
        return None
    else:
        version_regex = r"^version_str = ['\"]([^'\"]*)['\"]"
        mo = re.search(version_regex, version_string_line, re.M)
        if mo:
            return mo.group(1)


def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            paths.append(os.path.join('..', path, filename))
    return paths


VERSION_PY_FILENAME = 'kalliope/_version.py'
version = read_version_py(VERSION_PY_FILENAME)

extra_files = package_files('kalliope/trigger/snowboy')
extra_files.append('brain.yml')
extra_files.append('settings.yml')

setup(
    name='kalliope',
    version=version,
    description='Kalliope is a modular always-on voice controlled personal assistant designed for home automation.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/kalliope-project/kalliope',
    author='The dream team of Kalliope-project',
    author_email='kalliope-project@googlegroups.com',
    license='MIT',
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Home Automation',
        'Topic :: Multimedia :: Sound/Audio :: Speech',
        'Topic :: Multimedia :: Sound/Audio :: Sound Synthesis',
        'Topic :: Scientific/Engineering :: Artificial Intelligence'
    ],
    keywords='assistant bot TTS STT jarvis',
    zip_safe=False,
    # included packages
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    python_requires=">=3.6",
    # required libs
    install_requires=[
        'Werkzeug>=2.0',
        'pyyaml>=5.4.1',
        'six>=1.12.0',
        'SpeechRecognition>=3.8.1',
        'markupsafe>=1.1.1',
        'pyaudio>=0.2.11',
        'pyasn1>=0.4.5',
        'ansible>=4.5',
        'jinja2>=2.10.1',
        'cffi>=1.12.3',
        'ipaddress>=1.0.17',
        'flask>=2.0.1',
        'Flask-Restful>=0.3.7',
        'flask_cors>=3.0.8',
        'requests>=2.22.0',
        'httpretty>=0.8.14',
        'mock>=3.0.5',
        'Flask-Testing>=0.8.1',
        'apscheduler>=3.6.0',
        'GitPython>=2.1.11',
        'packaging>=19.0',
        'transitions>=0.6.9',
        'sounddevice>=0.3.13',
        'SoundFile>=0.10.2',
        'pyalsaaudio>=0.8.4',
        'paho-mqtt>=1.4.0',
        'voicerss_tts>=1.0.6',
        'gTTS>=2.2.3',
        'urllib3>=1.25.3',
        'gevent>=20.9.0'
    ],
    # additional files
    package_data={
        'kalliope': extra_files,
    },

    # entry point script
    entry_points={
        'console_scripts': [
            'kalliope=kalliope:main',
        ],
    },
)



import requests

from kalliope.core import FileManager
from kalliope.core.TTS.TTSModule import TTSModule, FailToLoadSoundFile, MissingTTSParameter
import logging

logging.basicConfig()
logger = logging.getLogger("kalliope")



class Marytts(TTSModule):
    def __init__(self, **kwargs):
        super(Marytts, self).__init__(**kwargs)
        self.host = kwargs.get('host', 'localhost')
        self.port = kwargs.get('port', '59125')
        self.locale = kwargs.get('locale', None)
        self.voice = kwargs.get('voice', None)
        self._check_parameters()

    def say(self, words):
        """
        :param words: The sentence to say
        """

        self.generate_and_play(words, self._generate_audio_file)

    def _check_parameters(self):
        """
        Check parameters are ok, raise MissingTTSParameterException exception otherwise.
        :return: true if parameters are ok, raise an exception otherwise

               .. raises:: MissingTTSParameterException
        """
        if self.locale is None or self.voice is None:
            raise MissingTTSParameter("[MaryTTS] Missing parameters, check documentation !")
        return True

    def _generate_audio_file(self):
        """
        Generic method used as a Callback in TTSModule
            - must provided the audio file and write it on the disk

        .. raises:: FailToLoadSoundFile
        """

        # Prepare payload
        payload = self.get_payload()

        # getting the audio
        r = requests.get('http://' + self.host + ':' + self.port + "/process?", payload)

        if r.status_code != 200:        
            raise FailToLoadSoundFile("MaryTTS : Fail while trying to remotely access the audio file")
        
        # OK we get the audio we can write the sound file
        FileManager.write_in_file(self.file_path, r.content)

    def get_payload(self):
        """
        Generic method used load the payload used to access the remote api

        :return: Payload to use to access the remote api
        """

        return {"INPUT_TEXT":self.words,
              "INPUT_TYPE":"TEXT",
              "LOCALE": self.locale,
              "VOICE":self.voice, 
              "OUTPUT_TYPE":"AUDIO",
              "AUDIO":"WAVE", 
              }


import unittest

import mock

from kalliope.signals.order import Order


class TestOrder(unittest.TestCase):

    def test_on_notification_received(self):
        testing_order = Order()

        # received valid notification skip_trigger
        notification = "skip_trigger"
        payload = {
            "status": True
        }
        with mock.patch("kalliope.signals.order.order.Order.switch_trigger") as mock_switch_trigger_method:
            testing_order.on_notification_received(notification=notification, payload=payload)
            mock_switch_trigger_method.assert_called_once_with(payload)

        # received valid notification skip_trigger_max_retry
        notification = "skip_trigger_max_retry"
        payload = {
            "max_retry": 5
        }
        with mock.patch("kalliope.signals.order.order.Order.set_counter_max_retry") as mock_set_counter_max_retry_method:
            testing_order.on_notification_received(notification=notification, payload=payload)
            mock_set_counter_max_retry_method.assert_called_once()

        # received valid notification skip_trigger_decrease_max_retry
        notification = "skip_trigger_decrease_max_retry"
        payload = None
        with mock.patch(
                "kalliope.signals.order.order.Order.decrease_max_retry") as mock_decrease_max_retry_method:
            testing_order.on_notification_received(notification=notification, payload=payload)
            mock_decrease_max_retry_method.assert_called_once()

    def test_set_counter_max_retry(self):

        # test valid payload
        testing_order = Order()
        payload = {
            "max_retry": 5
        }
        testing_order.set_counter_max_retry(payload)
        self.assertEqual(testing_order.counter_max_retry, 5)

        # test invalid payload
        testing_order = Order()
        payload = {
            "max_retry": -12
        }
        testing_order.set_counter_max_retry(payload)
        self.assertEqual(testing_order.counter_max_retry, 0)

        testing_order = Order()
        payload = {
            "wrong_key": 5
        }
        testing_order.set_counter_max_retry(payload)
        self.assertEqual(testing_order.counter_max_retry, 0)

    def decrease_max_retry(self):

        # counter should not move because 0 by default
        testing_order = Order()
        testing_order.decrease_max_retry()
        self.assertEqual(testing_order.counter_max_retry, 0)

        # update the counter
        testing_order = Order()
        testing_order.counter_max_retry = 5
        testing_order.decrease_max_retry()
        self.assertEqual(testing_order.counter_max_retry, 4)

        # update the counter and reach 0
        testing_order = Order()
        testing_order.skip_trigger = True
        testing_order.counter_max_retry = 1
        testing_order.decrease_max_retry()
        self.assertEqual(testing_order.counter_max_retry, 0)
        self.assertFalse(testing_order.skip_trigger)

    def test_switch_trigger(self):

        # valid payload
        testing_order = Order()
        payload = {
            "status": True
        }
        testing_order.switch_trigger(payload)
        self.assertTrue(testing_order.skip_trigger)

        testing_order = Order()
        payload = {
            "status": "True"
        }
        testing_order.switch_trigger(payload)
        self.assertTrue(testing_order.skip_trigger)

        testing_order = Order()
        payload = {
            "status": False
        }
        testing_order.switch_trigger(payload)
        self.assertFalse(testing_order.skip_trigger)

        testing_order = Order()
        payload = {
            "status": "False"
        }
        testing_order.switch_trigger(payload)
        self.assertFalse(testing_order.skip_trigger)

        # invalid payload
        testing_order = Order()
        payload = {
            "non-existing": "False"
        }
        testing_order.switch_trigger(payload)
        self.assertFalse(testing_order.skip_trigger)

        testing_order = Order()
        payload = {
            "status": "test"
        }
        testing_order.switch_trigger(payload)
        self.assertFalse(testing_order.skip_trigger)


if __name__ == '__main__':
    unittest.main()



import warnings

import requests
import urllib3
from gtts import gTTS
from kalliope.core.TTS.TTSModule import TTSModule, MissingTTSParameter
import logging

logging.basicConfig()
logger = logging.getLogger("kalliope")


class Googletts(TTSModule):
    def __init__(self, **kwargs):
        super(Googletts, self).__init__(**kwargs)

        self._check_parameters()

    def say(self, words):
        """
        :param words: The sentence to say
        """

        self.generate_and_play(words, self._generate_audio_file)

    def _check_parameters(self):
        """
        Check parameters are ok, raise MissingTTSParameterException exception otherwise.
        :return: true if parameters are ok, raise an exception otherwise

               .. raises:: MissingTTSParameterException
        """
        if self.language == "default" or self.language is None:
            raise MissingTTSParameter("[GoogleTTS] Missing parameters, check documentation !")
        return True

    def _generate_audio_file(self):
        """
        Generic method used as a Callback in TTSModule
            - must provided the audio file and write it on the disk

        .. raises:: FailToLoadSoundFile
        """

        # Since the gTTS lib disabled the SSL verification we get rid of insecure request warning
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        
        tts = gTTS(text=self.words, lang=self.language)
        
        # OK we get the audio we can write the sound file
        tts.save(self.file_path)
        
        # Re enable the warnings to avoid affecting the whole kalliope process 
        warnings.resetwarnings()
        


# coding: utf8
import logging
import os
import subprocess
import sndhdr

from kalliope.core.Utils.FileManager import FileManager

logging.basicConfig()
logger = logging.getLogger("kalliope")


class PlayerModule(object):
    """
    Mother class of Players. 
    Ability to convert sound files to wave format.
    """

    def __init__(self, **kwargs):

        # set parameter from what we receive from the settings
        self.convert = kwargs.get('convert_to_wav', True)

    @staticmethod
    def convert_mp3_to_wav(file_path_mp3):
        """
        API compatibility function as "mp3" is part of the method
        name, but the conversion via ffmpeg supports many other
        input formats as well. Please use convert_to_wav() instead.
        All invocations within kalliope haven been changed to call
        convert_to_wav(); this method invokes convert_to_wav() and
        remains only for compatiblity with plugins.
        :param file_path_mp3: the file path to convert to wav
        """
        return convert_to_wav(file_path=file_path_mp3)

    @staticmethod
    def convert_to_wav(file_path):
        """
        PyAudio, AlsaPlayer, sounddevices only support wav files.
        Other files must be converted to a wav in order to be played
        This function uses ffmpeg (which must be installed) for
        conversion; python core lib sndhdr is used to check if the
        provided file is already in wav format (skips conversion).
        :param file_path: the file path to convert to wav
        """
        filetype = sndhdr.whathdr(file_path)
        if filetype is None or filetype.filetype != 'wav':
            logger.debug("[PlayerModule] Converting file to wav file: %s" % file_path)
            fnull = open(os.devnull, 'w')
            # temp file
            tmp_file_wav = file_path+ ".wav"
            # Convert file to wave
            subprocess.call(['ffmpeg', '-y', '-i', file_path, tmp_file_wav],
                            stdout=fnull, stderr=fnull)
            # remove the original file
            FileManager.remove_file(file_path)
            # rename the temp file with the same name as the original file
            os.rename(tmp_file_wav, file_path)



from kalliope.core.Models.settings.SettingsEntry import SettingsEntry


class Options(SettingsEntry):
    """
    This Class is representing an Option element with parameters and values

    .. note:: must be defined in the settings.yml
    """

    def __init__(self, 
                 recognizer_multiplier=1, 
                 recognizer_energy_ratio=1.5, 
                 recognizer_recording_timeout=15.0, 
                 recognizer_recording_timeout_with_silence=3.0, 
                 deaf=None, 
                 mute=None):
    
        super(Options, self).__init__(name="Options")
        self.deaf = deaf
        self.mute = mute
        self.recognizer_multiplier = recognizer_multiplier
        self.recognizer_energy_ratio = recognizer_energy_ratio
        self.recognizer_recording_timeout = recognizer_recording_timeout
        self.recognizer_recording_timeout_with_silence = recognizer_recording_timeout_with_silence

    def __str__(self):
        return str(self.serialize())

    def serialize(self):
        return {
            'name': self.name,
            'recognizer_multiplier': self.recognizer_multiplier,
            'recognizer_energy_ratio': self.recognizer_energy_ratio,
            'recognizer_recording_timeout': self.recognizer_recording_timeout,
            'recognizer_recording_timeout_with_silence': self.recognizer_recording_timeout_with_silence,
            'deaf': self.deaf,
            'mute': self.mute
        }

    def __eq__(self, other):
        """
        This is used to compare 2 objects
        :param other: the Options to compare
        :return: True if both Options are similar, False otherwise
        """
        return self.__dict__ == other.__dict__




class Serialize(Exception):
    """
    When raised, the LIFO class return the current API response to the caller
    """
    pass


class SynapseListAddedToLIFO(Exception):
    """
    When raised, a synapse list to process has been added to the LIFO list.
    The LIFO must start over and process the last synapse list added
    """
    def __init__(self, message=None):
        # Call the base class constructor with the parameters it needs
        super(SynapseListAddedToLIFO, self).__init__(message)
        self.message = message
        print(message)